angular.module('aom')

  .filter('aomLongDateFtr', function (aomResourcesSrv) {
    return function (dateString) {
      if (dateString !== null) {
        var formattedDate = moment(dateString).format(aomResourcesSrv.data.resources.Formats.LongDateFormat);
        return formattedDate;
      } else {
        return null;
      }
    };
  })

  .filter('aomDateTimeFtr', function (aomResourcesSrv) {
    return function (dateString) {
      if (dateString !== null && dateString !== '') {
        var formattedDate = moment(dateString).format(aomResourcesSrv.data.resources.Formats.DateFormat + ' ' + aomResourcesSrv.data.resources.Formats.TimeFormat);
        return formattedDate;
      } else {
        return null;
      }
    };
  })

  .filter('aomDateFtr', function (aomResourcesSrv) {
    return function (date) {
      return date !== null && date !== '' ? moment(date).format(aomResourcesSrv.data.resources.Formats.DateFormat) : '';
    };
  })

  .filter('aomTimeFtr', function (aomResourcesSrv) {
    return function (time) {
      return time !== null && time !== '' ? moment(time).format(aomResourcesSrv.data.resources.Formats.TimeFormat) : '';
    };
  })

  .filter('aomDateAndTimeFtr', function (aomResourcesSrv) {
    return function (dateString) {
      if (dateString !== null && dateString !== '') {
        var formattedDate = moment(dateString).format(aomResourcesSrv.data.resources.Formats.DateTimeFormat);
        return formattedDate;
      } else {
        return null;
      }
    };
  })

  .filter('aomDateTimeDiffFtr', function (aomResourcesSrv) {
    return function (time) {
      var hms = time.split(':');
      return time !== null && time !== '' ? aomResourcesSrv.data.resources.Formats.DateTimeDiffFormat.replace(/\{H\}/, hms[0]).replace(/\{M\}/, hms[1]).replace(/\{S\}/, hms[2]) : '';
    };
  })

  .filter('aomLocationFtr', function () {
    return function (Location) {
      var locationString = '';
      if (Location !== null && Location.Latitude !== null && Location.Latitude !== '0' && Location.Longitude !== null && Location.Longitude !== '0') {
        locationString = '(' + Location.Latitude + ',' + Location.Longitude + ')';
      }
      return locationString;
    };
  })

  .filter('aomLinkFtr', function ($sce) {
    return function (text) {
      if (text !== null && text.length > 0) {
        var regexWeb = /href="([\S]+)"/g;
        var newString = text.replace(regexWeb, 'onClick="window.aomURLOpen(\'$1\'); return false;"');
        var regexPhone = /([ +])(\d{10,11}) /g;
        newString = newString.replace(regexPhone, ' <a onClick="window.aomURLOpen(\'tel:$2\'); return false;"> $2</a> ');
        return $sce.trustAsHtml(newString);
      }
      return '';
    };
  })

  .filter('aomSortFtr', function ($q, $filter, aomResourcesSrv) {
    return function (items, type) {
      // When restarting the system when the app comes online after an offline logon,
      // there is a delay before the appConfig is reloaded, so we need to check 
      // that the referenced resources have actually been initialised, if not simply return
      if (aomResourcesSrv.data.resources && aomResourcesSrv.data.resources.appConfig && aomResourcesSrv.data.resources.derived) {
        var sortKey;
        var sortControl = !aomResourcesSrv.data.resources.appConfig.SortControl;
        if (type === 'templates') {
          sortKey = 'Title';
        } else if (type === 'jobs') {
          sortKey = aomResourcesSrv.data.resources.derived.SelectedSort.Code;
          if (sortKey === 'Code') {
            sortKey = 'Title';
          }
        } else if (type === 'groups') {
          sortKey = 'Text';
          sortControl = false;
        }
        return $filter('orderBy')(items, sortKey, sortControl);
      }
    };
  })

  .filter('jobs', function ($filter) {
    return function (items, status) {
      return $filter('filter')(items, { jsProperties: { status: status } });
    };
  })

  .filter('url', function () {
    return function (input) {
      return encodeURI(input);
    };
  })

  .filter('unique', function () {
    return function (collection, keyname) {
      var output = [],
        keys = [];

      angular.forEach(collection, function (item) {
        var key = item[keyname];
        if (keys.indexOf(key) === -1) {
          keys.push(key);
          output.push(item);
        }
      });

      return output;
    };
  });
