﻿'use strict';
angular.module('aom.services')
  .factory('aomDatabaseSrv', function ($q, $timeout, $cordovaSQLite, $window, $ionicPlatform, aomNotificationSrv, aomResourcesSrv, aomPopupSrv, aomLogSrv, constants, appConfig) {
    var database = {};

    function runSqlCommands(obj) {
      var promises = [];
      for (var prop in obj) {
        if (obj.hasOwnProperty(prop)) {
          promises.push(
            $cordovaSQLite.execute(database.db, obj[prop])
              .finally(function () {
                return $q.when();
              })
          );
        }
      }
      return $q.all(promises);
    }

    function createTables() {
      return getSQLITEVersion()
        .then(function () {
          return runSqlCommands(constants.sql.tables);
        })
        .then(function () {
          if (appConfig.release) {
            constants.sql.adHoc.releaseURL = 'INSERT OR REPLACE INTO RootLink (id, link) values (1, \'' + appConfig.releaseURL + '\')';
          }
          return runSqlCommands(constants.sql.adHoc);
        });
    }

    function openCordovaDb() {
      $ionicPlatform.ready(function () {
        database.db = $cordovaSQLite.openDB({ name: 'aomv5.db', location: 'default' });
        createTables();
      });
    }

    function openWebDb() {
      database.db = window.openDatabase('aomv5.db', '1.0', 'AOM', -1);
      createTables();
    }

    function setupDb() {
      window.cordova ? openCordovaDb() : openWebDb();
    }

    function getSQLITEVersion() {
      var query = 'SELECT sqlite_version()';
      return $cordovaSQLite.execute(database.db, query).then(function (response) {
        if (response.rows.length > 0) {
          aomResourcesSrv.data.sqliteVersion = response.rows.item(0)['sqlite_version()'];
        }
      });
    }

    function whereClauseBuilder(whereObject) {
      var where = ' WHERE ',
        whereSpecified = false;

      for (var prop in whereObject) {
        if (whereObject.hasOwnProperty(prop)) {
          whereSpecified = true;
          var propValue = whereObject[prop];
          var clauseSpecified = propValue.substr(0, 1) === ';';
          if (clauseSpecified) {
            where += prop + ' ' + propValue.substr(1, propValue.length) + ' and ';
          } else {
            where += prop + ' = \'' + whereObject[prop] + '\' and ';
          }
        }
      }
      return whereSpecified ? where.substr(0, where.length - 5) : '';
    }

    setupDb();

    database.insertQuery = function (insertObject, replace, ignore) {
      var query = 'INSERT ',
        columns = ' (',
        values = ' VALUES (',
        fieldValues = [];

      query += replace ? 'OR REPLACE ' : '';
      query += ignore ? 'OR IGNORE ' : '';

      // TOOO handle bad object
      for (var table in insertObject) {
        if (insertObject.hasOwnProperty(table)) {
          query += ' INTO ' + table;
          var valuesObject = insertObject[table];
          if (valuesObject !== undefined) {
            for (var prop in valuesObject) {
              if (valuesObject.hasOwnProperty(prop)) {
                columns += prop + ', ';
                values += '?, ';
                fieldValues.push(prop.indexOf('_json') > -1 ? JSON.stringify(valuesObject[prop]) : valuesObject[prop]);
              }
            }
          }
        }
      }

      columns = columns.substr(0, columns.length - 2) + ')';
      values = values.substr(0, values.length - 2) + ')';
      query += columns + values;

      return $cordovaSQLite.execute(database.db, query, fieldValues)
        .catch(function (err) {
          aomLogSrv.log({ message: 'aomDatabaseSrv insert query failed', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
          return $q.when(err);
        });
    };

    database.updateQuery = function (updateObject) {
      var query = 'UPDATE ',
        where = '';

      for (var table in updateObject) {
        if (updateObject.hasOwnProperty(table)) {
          if (table === 'where') {
            where += whereClauseBuilder(updateObject.where);
          } else if (updateObject !== undefined) {
            query += table + ' SET ';
            var valuesObject = updateObject[table];
            if (valuesObject !== undefined) {
              for (var prop in valuesObject) {
                if (valuesObject.hasOwnProperty(prop)) {
                  query += prop + ' = \'' + valuesObject[prop] + '\', ';
                }
              }
            }
          }
        }
      }

      query = query.substr(0, query.length - 2) + where;

      return $cordovaSQLite.execute(database.db, query)
        .catch(function (err) {
          aomLogSrv.log({ message: 'aomDatabaseSrv update query failed', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
          return $q.when(err);
        });
    };

    database.deleteQuery = function (deleteObject) {
      var query = 'DELETE FROM ';

      for (var table in deleteObject) {
        if (deleteObject.hasOwnProperty(table)) {
          query += table;
          var valuesObject = deleteObject[table];
          query += whereClauseBuilder(valuesObject);
        }
      }

      return $cordovaSQLite.execute(database.db, query)
        .catch(function (err) {
          aomLogSrv.log({ message: 'aomDatabaseSrv delete query failed', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
          return $q.when(err);
        });
    };

    database.selectQuery = function (selectObject) {
      var query = 'SELECT ',
        jsonFields = [],
        columnCount = 0,
        singleColumn = null,
        columnsSpecified = false,
        where = '';

      for (var table in selectObject) {
        if (selectObject.hasOwnProperty(table)) {
          var fromObject = selectObject[table];
          if (table === 'where') {
            where += whereClauseBuilder(fromObject);
          } else if (fromObject !== undefined) {
            for (var prop in fromObject) {
              if (fromObject.hasOwnProperty(prop)) {
                singleColumn = prop;
                query += prop + ', ';
                columnCount += 1;
                columnsSpecified = true;
              }
            }
            query = columnsSpecified ? query.substr(0, query.length - 2) : query + '*';
            query += ' FROM ' + table;
          }
        }
      }
      query += where;

      return $cordovaSQLite.execute(database.db, query)
        .then(function (res) {
          var returnData = { rows: [] };
          if (res.rows.length > 0) {
            // Populate and json columns where required
            var rowZero = res.rows.item(0);
            for (var prop in rowZero) {
              if (rowZero.hasOwnProperty(prop)) {
                if (prop.indexOf('_json') > -1) {
                  jsonFields.push(prop);
                }
              }
            }
            // Loop over the rows returned
            for (var i = 0; i < res.rows.length; i++) {
              var thisRecord = res.rows.item(i);

              // For JSON data rows
              for (var x = 0; x < jsonFields.length; x++) {
                thisRecord[jsonFields[x]] = JSON.parse(thisRecord[jsonFields[x]]);
              }
              // If there is only one column specified then return it in the array rather than in the object
              // We need this for jobs but perhaps We should change jobs
              if (columnsSpecified && columnCount === 1) {
                returnData.rows.push(thisRecord[singleColumn]);
              } else {
                returnData.rows.push(thisRecord);
              }
            }
          }
          return $q.when(returnData);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'aomDatabaseSrv select query failed', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
          return $q.reject(err);
        });
    };

    return database;
  });
