'use strict';
angular.module('aom.directives')
  .directive('aomSearchMediaList', function () {
    return {
      scope: {
        goBack: '='
      },
      controller: function ($scope, $ionicModal, $sce, $ionicHistory, aomResourcesSrv, aomBackSrv, aomJobSrv, aomAPISrv, aomCleanSrv, constants, aomFileSystemSrv, aomMediaSrv, aomHardwareSrv) {

        $scope.data = {
          resources: aomResourcesSrv.data.resources,
          attachments: [],
          field: aomJobSrv.data.selectedField,
          links: {}
        };

        $scope.backToJob = function backToJob() {
          $scope.goBack();
          aomBackSrv.exit(true);
        };

        function showMedia(type) {
          var templateUrl = 'App/Common/Templates/' + type + '-popover.html';
          $ionicModal.fromTemplateUrl(templateUrl, {
            scope: $scope,
            animation: aomHardwareSrv.isTomTom() ? 'none' : 'slide-in-up'
          }).then(function (modal) {
            $scope.modal = modal;
            $scope.modal.show();
          });
        }

        function openFile(attachment) {
          if (attachment.fileType === 'image') {
            $scope.imageUrl = aomFileSystemSrv.adhocDocumentsPathNormalized + '/' + attachment.LinkReference + attachment.Extension;
            showMedia(attachment.fileType);
          } else if (attachment.fileType === 'video') {
            $scope.videoUrl = $sce.trustAsResourceUrl(aomFileSystemSrv.adhocDocumentsPathNormalized + '/' + attachment.LinkReference + attachment.Extension + '#t=1');
            showMedia(attachment.fileType);
          } else {
            aomFileSystemSrv.openSystemViewer(attachment.LinkReference + attachment.Extension, attachment.MimeType, constants.common.aomAdhocAttachmentsDirectory);
          }
        }

        $scope.getAttachments = function () {
          if (aomJobSrv.data.selectedField.Data.length > 0) {
            var templateEngine = urltemplate.parse(aomAPISrv.data.rootLinks.AttachmentSearchByTerm.href);
            var url = templateEngine.expand({
              term: aomJobSrv.data.selectedField.Data,
              applicationReference: aomJobSrv.data.selectedJob.ApplicationReference
            });
            aomAPISrv.getData(url)
              .then(function (response) {
                response.Results.map(function (attachment) {
                  attachment.fileType = aomMediaSrv.getFileType(attachment.MimeType);
                });
                $scope.data.links = response.links.attachments;
                $scope.data.attachments = response.Results;
              });
          }
        };

        $scope.getAttachments();

        $scope.downloadAndOpenFile = function (attachment) {
          if (attachment.status !== aomResourcesSrv.data.resources.Media.Downloading) {
            if (attachment.status !== aomResourcesSrv.data.resources.Media.Downloaded) {
              attachment.status = aomResourcesSrv.data.resources.Media.Downloading;
            }
            aomCleanSrv.getAttachment($scope.data.links[attachment.LinkReference], constants.common.aomAdhocAttachmentsDirectory)
              .then(function () {
                aomJobSrv.data.downloadedAdhocAttachment = true;
                attachment.status = aomResourcesSrv.data.resources.Media.Downloaded;
                openFile(attachment);
              })
              .catch(function () {
                attachment.status = aomResourcesSrv.data.resources.Media.Failed;
              });
          }
        };

        $scope.closeModal = function () {
          $scope.modal.hide()
            .then(function () {
              $scope.modal.remove();
            });
        };

        this.$onInit = function () {
          aomBackSrv.init($scope.backToJob);
        };

      },
      templateUrl: 'App/Components/SearchMediaList/searchMediaList.html'
    };
  });